function [ac] = cal_ac(x,sr,mapping,b2,a2,t2,n_s)
%function [ac] = cal_ac(x,sr,mapping,fmin,bpo,q,n,ftype)
% function [ac] = cal_ac(x,sr,mapping,fmin,bpo,q,n,ftype)
%
% Calculate normalized subband autocorrelation. If 'mapping' exists, do
% post-processing using k-dim PCA dimensionality reduction.
%
% INPUT:
%         x - the input signal
%         sr - the sampling rate
%         mapping - (nchs x 1) cell containing subband PCAs (maxlags x kdim)
%         fmin - the center frequency of the minimum frequency band
%         bpo - band-per-octave
%         q - the constant Q-factor of the filter
%         n - the order of filter
%         ftype - filter type
%
% OUTPUT:
%         ac - normalized subband autocorrelation
%                  (nchs X maxlags X nfrms)
%              if 'mapping' exists, do k-dim PCA dimensionality reduction
%                  (nchs X kdim X nfrms)
%         x - the input signal (one channel)
%         acf - subband autocorrelation (not used for longer audio)
%         acf_engy - energy of subband autocorrelation (not used for longer audio)
%
% [2012-04-11] Byung Suk Lee bsl@ee.columbia.edu

flag_mapping =  exist('mapping','var') && ~isempty(mapping);
if flag_mapping, nchs = size(mapping,1); else nchs = 48; end;
if flag_mapping, maxlags = size(mapping{1},1); else maxlags = sr*0.025; end;
if flag_mapping, kdim = size(mapping{1},2); else kdim = maxlags; end;
if nargin < 4
if ~exist('fmin','var'), fmin = 100; end;
if ~exist('bpo','var'), bpo = 16; end;
if ~exist('q','var'), q = 8; end;
if ~exist('n','var'), n = 2; end;
if ~exist('ftype','var'), ftype = 2; end;
% Design Lyon-Slaney filters
[b2,a2,t2,w2,cf] = bpfiltbank(sr,fmin,bpo,nchs,q,n,ftype);  % type 2 is LySla
end
if nargin < 7, n_s = 5; end;

% Use only mono channel
if size(x,2) > 1, x = x(:,1); end;

% Design Lyon-Slaney filters
%[b2,a2,t2,w2,cf] = bpfiltbank(sr,fmin,bpo,nchs,q,n,ftype);  % type 2 is LySla

%%
nfrms = 1 + max(floor((size(x,1) - 0.025*sr)/(0.01*sr)),0);

%% Dithering
x = dithering(x);
xlen = length(x);
fps = 100;

%% calculate sbpca features for the audio at least the length of the analysis window, 25 ms
disp(['Autocorrelogram (maxlags:',num2str(maxlags),', nchs:',num2str(nchs),', kdim:',num2str(kdim),')'])
if xlen >= 0.025*sr

buf_len = round(0.1*sr); % buffer length of front and end of slice
nslice = max(ceil(length(x) / (n_s * sr))-1,1);
ac = zeros(nfrms,nchs*kdim);
%% feature calculation on slices of length n_s sec
for islice = 1:nslice
    ids = n_s*sr*(islice-1) - buf_len * (islice > 1) + 1; % front buffer, except the first slice
%     ide = min( n_s*sr*islice + buf_len, xlen); % end buffer, always
    tide = min( n_s*sr*islice + buf_len, xlen);
    if islice == nslice
        ide = xlen;
    else
        ide = tide; % end buffer, always
    end
    disp(['(',num2str(islice),'/',num2str(nslice),') Start: ',num2str(ids),' End: ',num2str(ide)]);
    xx = x(ids:ide);
    if flag_mapping
        [tmpac] = sub_cal_ac(xx,sr,b2,a2,t2,mapping);
    else
        [tmpac] = sub_cal_ac(xx,sr,b2,a2,t2);
    end
    % cut frount and end buffer results
    tmpac = tmpac(:,:,(1+10*(islice > 1)):(end-8*(islice < nslice)));
    if islice == nslice
        lac = nfrms - fps*n_s * (nslice - 1);
    else
        lac = size(tmpac,3);
    end
    disp(['islice: ',num2str(islice),' len_tmpac: ',num2str(size(tmpac,3)),' lac: ',num2str(lac)]);
    tmpac = reshape(tmpac,[nchs*kdim lac]);
    ac((1:lac) + n_s*fps*(islice - 1),:) = tmpac';
end


else
    err_txt = 'Not calculating features: ';
    err_msg = 'Input audio is shorter than the analysis window, 25 ms.';
    disp([err_txt,err_msg]);
    disp(['Zero features of ',num2str(nfrms),' frames are saved.']);
    ac = zeros(nfrms,nchs*kdim);
end


function [ac, x, acf, acf_engy] = sub_cal_ac(x,sr,b2,a2,t2,mapping)
% function [ac, x, acf, acf_engy] = sub_cal_ac(x,sr,b2,a2,t2,mapping)
%
% Calculate normalized subband autocorrelation. If 'mapping' exists, do
% post-processing using k-dim PCA dimensionality reduction.
%
% INPUT:
%         x - The input signal
%         sr - The sampling rate
%         b2 - B coefficients to IIR filterbank
%         a2 - A coefficients to IIR filterbank
%         t2 - Delay coefficients to IIR filterbank
%         mapping - (nchs x 1) cell containing subband PCA mapping (maxlags x kdim)
%
% OUTPUT:
%         ac - normalized subband autocorrelation
%                  (nchs X maxlags X nfrms)
%              if 'mapping' exists, do k-dim PCA dimensionality reduction
%                  (nchs X kdim X nfrms)
%              nfrms = 1 + max(floor((size(x,1) - 0.025*sr)/(0.01*sr)),0)
%         x - the input signal (one channel)
%         acf - subband autocorrelation (not used for longer audio)
%         acf_engy - energy of subband autocorrelation (not used for longer audio)
%

flag_mapping =  exist('mapping','var') && ~isempty(mapping);

if flag_mapping, maxlags = size(mapping{1},1); else maxlags = sr*0.025; end;
if flag_mapping, kdim = size(mapping{1},2); else kdim = maxlags; end;
nchs = length(t2);

margin = floor(0.04*sr);
nfrms = 1 + max(floor((size(x,1) - 0.025*sr)/(0.01*sr)),0);

if size(x,1) < 0.025*sr
    
    acf = zeros(nchs,maxlags,1);
    acf_engy = ones(nchs,maxlags,1);

else
    x = [x;zeros(margin,1)];

    % cochlear filtering
    x = x(:)';
    e = filterbank(b2,a2,x,1,0,t2);

    [c1, s1, lags] = autocorrelogram(e, sr, maxlags); % multichannel autocorrelation

    acf = c1(:,:,1:nfrms);
    acf_engy = s1(:,:,1:nfrms);
end

ac = acf ./ (acf_engy + (acf_engy == 0));

if flag_mapping
    kac = zeros(nchs,kdim,nfrms);
    for c = 1:nchs
        tmpac = shiftdim(ac(c,:,:),1);
        %% To ignore frames that contain NaN elements
        sumtmpac = sum(tmpac);
        tmpac(:,isnan(sumtmpac)) = 0;
        kac(c,:,:) = mapping{c}' * tmpac;
    end
    ac = kac;
end

